@@ -1,7 +1,7 @@
 # !/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,31 +16,33 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-import sys
+# pylint: disable=E0401, C0114
 import subprocess
-
+import sys
+from typing import Optional
 
 
 from . import __version__ as telegram_ver
 
 
-def _git_revision():
+def _git_revision() -> Optional[str]:
     try:
-        output = subprocess.check_output(["git", "describe", "--long", "--tags"],
-                                         stderr=subprocess.STDOUT)
+        output = subprocess.check_output(
+            ["git", "describe", "--long", "--tags"], stderr=subprocess.STDOUT
+        )
     except (subprocess.SubprocessError, OSError):
         return None
     return output.decode().strip()
 
 
-def print_ver_info():
+def print_ver_info() -> None:
     git_revision = _git_revision()
-    print('python-telegram-bot {}'.format(telegram_ver) + (' ({})'.format(git_revision)
-                                                           if git_revision else ''))
-    print('Python {}'.format(sys.version.replace('\n', ' ')))
+    print(f'python-telegram-bot {telegram_ver}' + (f' ({git_revision})' if git_revision else ''))
+    sys_version = sys.version.replace('\n', ' ')
+    print(f'Python {sys_version}')
 
 
-def main():
+def main() -> None:
     print_ver_info()
 
 
