@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2018
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,29 +16,31 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
+# pylint: disable=C0115
 """This module contains an object that represents Telegram errors."""
+from typing import Tuple
 
 
-def _lstrip_str(in_s, lstr):
+def _lstrip_str(in_s: str, lstr: str) -> str:
     """
     Args:
         in_s (:obj:`str`): in string
         lstr (:obj:`str`): substr to strip from left side
 
     Returns:
-        str:
+        :obj:`str`: The stripped string.
 
     """
     if in_s.startswith(lstr):
-        res = in_s[len(lstr):]
+        res = in_s[len(lstr) :]
     else:
         res = in_s
     return res
 
 
 class TelegramError(Exception):
-    def __init__(self, message):
-        super(TelegramError, self).__init__()
+    def __init__(self, message: str):
+        super().__init__()
 
         msg = _lstrip_str(message, 'Error: ')
         msg = _lstrip_str(msg, '[Error]: ')
@@ -48,8 +50,11 @@ class TelegramError(Exception):
             msg = msg.capitalize()
         self.message = msg
 
-    def __str__(self):
-        return '%s' % (self.message)
+    def __str__(self) -> str:
+        return '%s' % self.message
+
+    def __reduce__(self) -> Tuple[type, Tuple[str]]:
+        return self.__class__, (self.message,)
 
 
 class Unauthorized(TelegramError):
@@ -57,8 +62,11 @@ class Unauthorized(TelegramError):
 
 
 class InvalidToken(TelegramError):
-    def __init__(self):
-        super(InvalidToken, self).__init__('Invalid token')
+    def __init__(self) -> None:
+        super().__init__('Invalid token')
+
+    def __reduce__(self) -> Tuple[type, Tuple]:  # type: ignore[override]
+        return self.__class__, ()
 
 
 class NetworkError(TelegramError):
@@ -70,44 +78,51 @@ class BadRequest(NetworkError):
 
 
 class TimedOut(NetworkError):
-    def __init__(self):
-        super(TimedOut, self).__init__('Timed out')
+    def __init__(self) -> None:
+        super().__init__('Timed out')
+
+    def __reduce__(self) -> Tuple[type, Tuple]:  # type: ignore[override]
+        return self.__class__, ()
 
 
 class ChatMigrated(TelegramError):
     """
     Args:
-        new_chat_id (:obj:`int`):
+        new_chat_id (:obj:`int`): The new chat id of the group.
 
     """
 
-    def __init__(self, new_chat_id):
-        super(ChatMigrated,
-              self).__init__('Group migrated to supergroup. New chat id: {}'.format(new_chat_id))
+    def __init__(self, new_chat_id: int):
+        super().__init__(f'Group migrated to supergroup. New chat id: {new_chat_id}')
         self.new_chat_id = new_chat_id
 
+    def __reduce__(self) -> Tuple[type, Tuple[int]]:  # type: ignore[override]
+        return self.__class__, (self.new_chat_id,)
+
 
 class RetryAfter(TelegramError):
     """
     Args:
-        retry_after (:obj:`int`):
+        retry_after (:obj:`int`): Time in seconds, after which the bot can retry the request.
 
     """
 
-    def __init__(self, retry_after):
-        super(RetryAfter,
-              self).__init__('Flood control exceeded. Retry in {} seconds'.format(retry_after))
+    def __init__(self, retry_after: int):
+        super().__init__(f'Flood control exceeded. Retry in {float(retry_after)} seconds')
         self.retry_after = float(retry_after)
 
+    def __reduce__(self) -> Tuple[type, Tuple[float]]:  # type: ignore[override]
+        return self.__class__, (self.retry_after,)
+
 
 class Conflict(TelegramError):
     """
-        Raised when a long poll or webhook conflicts with another one.
+    Raised when a long poll or webhook conflicts with another one.
 
-        Args:
-            msg (:obj:`str`): The message from telegrams server.
+    Args:
+        msg (:obj:`str`): The message from telegrams server.
 
     """
 
-    def __init__(self, msg):
-        super(Conflict, self).__init__(msg)
+    def __reduce__(self) -> Tuple[type, Tuple[str]]:
+        return self.__class__, (self.message,)
