@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -19,35 +19,48 @@
 """This module contains the DictPersistence class."""
 from copy import deepcopy
 
-from telegram.utils.helpers import decode_user_chat_data_from_json,\
-    decode_conversations_from_json, encode_conversations_to_json
+from typing import Any, DefaultDict, Dict, Optional, Tuple
+from collections import defaultdict
+
+from telegram.utils.helpers import (
+    decode_conversations_from_json,
+    decode_user_chat_data_from_json,
+    encode_conversations_to_json,
+)
+from telegram.ext import BasePersistence
+from telegram.utils.types import ConversationDict
 
 try:
     import ujson as json
 except ImportError:
-    import json
-from collections import defaultdict
-from telegram.ext import BasePersistence
+    import json  # type: ignore[no-redef]
 
 
 class DictPersistence(BasePersistence):
     """Using python's dicts and json for making your bot persistent.
 
-    Attributes:
-        store_user_data (:obj:`bool`): Whether user_data should be saved by this
-            persistence class.
-        store_chat_data (:obj:`bool`): Whether chat_data should be saved by this
-            persistence class.
-        store_bot_data (:obj:`bool`): Whether bot_data should be saved by this
-            persistence class.
+    Note:
+        This class does *not* implement a :meth:`flush` method, meaning that data managed by
+        ``DictPersistence`` is in-memory only and will be lost when the bot shuts down. This is,
+        because ``DictPersistence`` is mainly intended as starting point for custom persistence
+        classes that need to JSON-serialize the stored data before writing them to file/database.
+
+    Warning:
+        :class:`DictPersistence` will try to replace :class:`telegram.Bot` instances by
+        :attr:`REPLACED_BOT` and insert the bot set with
+        :meth:`telegram.ext.BasePersistence.set_bot` upon loading of the data. This is to ensure
+        that changes to the bot apply to the saved objects, too. If you change the bots token, this
+        may lead to e.g. ``Chat not found`` errors. For the limitations on replacing bots see
+        :meth:`telegram.ext.BasePersistence.replace_bot` and
+        :meth:`telegram.ext.BasePersistence.insert_bot`.
 
     Args:
         store_user_data (:obj:`bool`, optional): Whether user_data should be saved by this
-            persistence class. Default is ``True``.
+            persistence class. Default is :obj:`True`.
         store_chat_data (:obj:`bool`, optional): Whether user_data should be saved by this
-            persistence class. Default is ``True``.
+            persistence class. Default is :obj:`True`.
         store_bot_data (:obj:`bool`, optional): Whether bot_data should be saved by this
-            persistence class. Default is ``True`` .
+            persistence class. Default is :obj:`True` .
         user_data_json (:obj:`str`, optional): Json string that will be used to reconstruct
             user_data on creating this persistence. Default is ``""``.
         chat_data_json (:obj:`str`, optional): Json string that will be used to reconstruct
@@ -56,19 +69,31 @@ class DictPersistence(BasePersistence):
             bot_data on creating this persistence. Default is ``""``.
         conversations_json (:obj:`str`, optional): Json string that will be used to reconstruct
             conversation on creating this persistence. Default is ``""``.
+
+    Attributes:
+        store_user_data (:obj:`bool`): Whether user_data should be saved by this
+            persistence class.
+        store_chat_data (:obj:`bool`): Whether chat_data should be saved by this
+            persistence class.
+        store_bot_data (:obj:`bool`): Whether bot_data should be saved by this
+            persistence class.
     """
 
-    def __init__(self,
-                 store_user_data=True,
-                 store_chat_data=True,
-                 store_bot_data=True,
-                 user_data_json='',
-                 chat_data_json='',
-                 bot_data_json='',
-                 conversations_json=''):
-        super().__init__(store_user_data=store_user_data,
-                         store_chat_data=store_chat_data,
-                         store_bot_data=store_bot_data)
+    def __init__(
+        self,
+        store_user_data: bool = True,
+        store_chat_data: bool = True,
+        store_bot_data: bool = True,
+        user_data_json: str = '',
+        chat_data_json: str = '',
+        bot_data_json: str = '',
+        conversations_json: str = '',
+    ):
+        super().__init__(
+            store_user_data=store_user_data,
+            store_chat_data=store_chat_data,
+            store_bot_data=store_bot_data,
+        )
         self._user_data = None
         self._chat_data = None
         self._bot_data = None
@@ -81,20 +106,20 @@ class DictPersistence(BasePersistence):
             try:
                 self._user_data = decode_user_chat_data_from_json(user_data_json)
                 self._user_data_json = user_data_json
-            except (ValueError, AttributeError):
-                raise TypeError("Unable to deserialize user_data_json. Not valid JSON")
+            except (ValueError, AttributeError) as exc:
+                raise TypeError("Unable to deserialize user_data_json. Not valid JSON") from exc
         if chat_data_json:
             try:
                 self._chat_data = decode_user_chat_data_from_json(chat_data_json)
                 self._chat_data_json = chat_data_json
-            except (ValueError, AttributeError):
-                raise TypeError("Unable to deserialize chat_data_json. Not valid JSON")
+            except (ValueError, AttributeError) as exc:
+                raise TypeError("Unable to deserialize chat_data_json. Not valid JSON") from exc
         if bot_data_json:
             try:
                 self._bot_data = json.loads(bot_data_json)
                 self._bot_data_json = bot_data_json
-            except (ValueError, AttributeError):
-                raise TypeError("Unable to deserialize bot_data_json. Not valid JSON")
+            except (ValueError, AttributeError) as exc:
+                raise TypeError("Unable to deserialize bot_data_json. Not valid JSON") from exc
             if not isinstance(self._bot_data, dict):
                 raise TypeError("bot_data_json must be serialized dict")
 
@@ -102,63 +127,62 @@ class DictPersistence(BasePersistence):
             try:
                 self._conversations = decode_conversations_from_json(conversations_json)
                 self._conversations_json = conversations_json
-            except (ValueError, AttributeError):
-                raise TypeError("Unable to deserialize conversations_json. Not valid JSON")
+            except (ValueError, AttributeError) as exc:
+                raise TypeError(
+                    "Unable to deserialize conversations_json. Not valid JSON"
+                ) from exc
 
     @property
-    def user_data(self):
-        """:obj:`dict`: The user_data as a dict"""
+    def user_data(self) -> Optional[DefaultDict[int, Dict]]:
+        """:obj:`dict`: The user_data as a dict."""
         return self._user_data
 
     @property
-    def user_data_json(self):
+    def user_data_json(self) -> str:
         """:obj:`str`: The user_data serialized as a JSON-string."""
         if self._user_data_json:
             return self._user_data_json
-        else:
-            return json.dumps(self.user_data)
+        return json.dumps(self.user_data)
 
     @property
-    def chat_data(self):
-        """:obj:`dict`: The chat_data as a dict"""
+    def chat_data(self) -> Optional[DefaultDict[int, Dict]]:
+        """:obj:`dict`: The chat_data as a dict."""
         return self._chat_data
 
     @property
-    def chat_data_json(self):
+    def chat_data_json(self) -> str:
         """:obj:`str`: The chat_data serialized as a JSON-string."""
         if self._chat_data_json:
             return self._chat_data_json
-        else:
-            return json.dumps(self.chat_data)
+        return json.dumps(self.chat_data)
 
     @property
-    def bot_data(self):
-        """:obj:`dict`: The bot_data as a dict"""
+    def bot_data(self) -> Optional[Dict]:
+        """:obj:`dict`: The bot_data as a dict."""
         return self._bot_data
 
     @property
-    def bot_data_json(self):
+    def bot_data_json(self) -> str:
         """:obj:`str`: The bot_data serialized as a JSON-string."""
         if self._bot_data_json:
             return self._bot_data_json
-        else:
-            return json.dumps(self.bot_data)
+        return json.dumps(self.bot_data)
 
     @property
-    def conversations(self):
-        """:obj:`dict`: The conversations as a dict"""
+    def conversations(self) -> Optional[Dict[str, Dict[Tuple, Any]]]:
+        """:obj:`dict`: The conversations as a dict."""
         return self._conversations
 
     @property
-    def conversations_json(self):
+    def conversations_json(self) -> str:
         """:obj:`str`: The conversations serialized as a JSON-string."""
         if self._conversations_json:
             return self._conversations_json
-        else:
-            return encode_conversations_to_json(self.conversations)
+        return encode_conversations_to_json(self.conversations)  # type: ignore[arg-type]
 
-    def get_user_data(self):
-        """Returns the user_data created from the ``user_data_json`` or an empty defaultdict.
+    def get_user_data(self) -> DefaultDict[int, Dict[Any, Any]]:
+        """Returns the user_data created from the ``user_data_json`` or an empty
+        :obj:`defaultdict`.
 
         Returns:
             :obj:`defaultdict`: The restored user data.
@@ -167,59 +191,64 @@ class DictPersistence(BasePersistence):
             pass
         else:
             self._user_data = defaultdict(dict)
-        return deepcopy(self.user_data)
+        return deepcopy(self.user_data)  # type: ignore[arg-type]
 
-    def get_chat_data(self):
-        """Returns the chat_data created from the ``chat_data_json`` or an empty defaultdict.
+    def get_chat_data(self) -> DefaultDict[int, Dict[Any, Any]]:
+        """Returns the chat_data created from the ``chat_data_json`` or an empty
+        :obj:`defaultdict`.
 
         Returns:
-            :obj:`defaultdict`: The restored user data.
+            :obj:`defaultdict`: The restored chat data.
         """
         if self.chat_data:
             pass
         else:
             self._chat_data = defaultdict(dict)
-        return deepcopy(self.chat_data)
+        return deepcopy(self.chat_data)  # type: ignore[arg-type]
 
-    def get_bot_data(self):
-        """Returns the bot_data created from the ``bot_data_json`` or an empty dict.
+    def get_bot_data(self) -> Dict[Any, Any]:
+        """Returns the bot_data created from the ``bot_data_json`` or an empty :obj:`dict`.
 
         Returns:
-            :obj:`defaultdict`: The restored user data.
+            :obj:`dict`: The restored bot data.
         """
         if self.bot_data:
             pass
         else:
             self._bot_data = {}
-        return deepcopy(self.bot_data)
+        return deepcopy(self.bot_data)  # type: ignore[arg-type]
 
-    def get_conversations(self, name):
+    def get_conversations(self, name: str) -> ConversationDict:
         """Returns the conversations created from the ``conversations_json`` or an empty
-        defaultdict.
+        :obj:`dict`.
 
         Returns:
-            :obj:`defaultdict`: The restored user data.
+            :obj:`dict`: The restored conversations data.
         """
         if self.conversations:
             pass
         else:
             self._conversations = {}
-        return self.conversations.get(name, {}).copy()
+        return self.conversations.get(name, {}).copy()  # type: ignore[union-attr]
 
-    def update_conversation(self, name, key, new_state):
+    def update_conversation(
+        self, name: str, key: Tuple[int, ...], new_state: Optional[object]
+    ) -> None:
         """Will update the conversations for the given handler.
 
         Args:
-            name (:obj:`str`): The handlers name.
+            name (:obj:`str`): The handler's name.
             key (:obj:`tuple`): The key the state is changed for.
             new_state (:obj:`tuple` | :obj:`any`): The new state for the given key.
         """
+        if not self._conversations:
+            self._conversations = {}
         if self._conversations.setdefault(name, {}).get(key) == new_state:
             return
         self._conversations[name][key] = new_state
         self._conversations_json = None
 
-    def update_user_data(self, user_id, data):
+    def update_user_data(self, user_id: int, data: Dict) -> None:
         """Will update the user_data (if changed).
 
         Args:
@@ -233,7 +262,7 @@ class DictPersistence(BasePersistence):
         self._user_data[user_id] = data
         self._user_data_json = None
 
-    def update_chat_data(self, chat_id, data):
+    def update_chat_data(self, chat_id: int, data: Dict) -> None:
         """Will update the chat_data (if changed).
 
         Args:
@@ -247,7 +276,7 @@ class DictPersistence(BasePersistence):
         self._chat_data[chat_id] = data
         self._chat_data_json = None
 
-    def update_bot_data(self, data):
+    def update_bot_data(self, data: Dict) -> None:
         """Will update the bot_data (if changed).
 
         Args:
