@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,8 +18,13 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains the classes that represent Telegram InlineQueryResultCachedVoice."""
 
-from telegram import InlineQueryResult
-from telegram.utils.helpers import DEFAULT_NONE
+from typing import TYPE_CHECKING, Any, Union, Tuple, List
+
+from telegram import InlineQueryResult, MessageEntity
+from telegram.utils.helpers import DEFAULT_NONE, DefaultValue
+
+if TYPE_CHECKING:
+    from telegram import InputMessageContent, ReplyMarkup
 
 
 class InlineQueryResultCachedVoice(InlineQueryResult):
@@ -28,6 +33,23 @@ class InlineQueryResultCachedVoice(InlineQueryResult):
     message will be sent by the user. Alternatively, you can use :attr:`input_message_content` to
     send a message with the specified content instead of the voice message.
 
+    Args:
+        id (:obj:`str`): Unique identifier for this result, 1-64 bytes.
+        voice_file_id (:obj:`str`): A valid file identifier for the voice message.
+        title (:obj:`str`): Voice message title.
+        caption (:obj:`str`, optional): Caption, 0-1024 characters after entities parsing.
+        parse_mode (:obj:`str`, optional): Send Markdown or HTML, if you want Telegram apps to show
+            bold, italic, fixed-width text or inline URLs in the media caption. See the constants
+            in :class:`telegram.ParseMode` for the available modes.
+        caption_entities (List[:class:`telegram.MessageEntity`], optional): List of special
+            entities that appear in the caption, which can be specified instead of
+            :attr:`parse_mode`.
+        reply_markup (:class:`telegram.InlineKeyboardMarkup`, optional): Inline keyboard attached
+            to the message.
+        input_message_content (:class:`telegram.InputMessageContent`, optional): Content of the
+            message to be sent instead of the voice message.
+        **kwargs (:obj:`dict`): Arbitrary keyword arguments.
+
     Attributes:
         type (:obj:`str`): 'voice'.
         id (:obj:`str`): Unique identifier for this result, 1-64 bytes.
@@ -37,36 +59,28 @@ class InlineQueryResultCachedVoice(InlineQueryResult):
         parse_mode (:obj:`str`): Optional. Send Markdown or HTML, if you want Telegram apps to show
             bold, italic, fixed-width text or inline URLs in the media caption. See the constants
             in :class:`telegram.ParseMode` for the available modes.
+        caption_entities (List[:class:`telegram.MessageEntity`]): Optional. List of special
+            entities that appear in the caption, which can be specified instead of
+            :attr:`parse_mode`.
         reply_markup (:class:`telegram.InlineKeyboardMarkup`): Optional. Inline keyboard attached
             to the message.
         input_message_content (:class:`telegram.InputMessageContent`): Optional. Content of the
-            message to be sent instead of the voice.
-
-    Args:
-        id (:obj:`str`): Unique identifier for this result, 1-64 bytes.
-        voice_file_id (:obj:`str`): A valid file identifier for the voice message.
-        title (:obj:`str`): Voice message title.
-        caption (:obj:`str`, optional): Caption, 0-1024 characters after entities parsing.
-        parse_mode (:obj:`str`, optional): Send Markdown or HTML, if you want Telegram apps to show
-            bold, italic, fixed-width text or inline URLs in the media caption. See the constants
-            in :class:`telegram.ParseMode` for the available modes.
-        reply_markup (:class:`telegram.InlineKeyboardMarkup`, optional): Inline keyboard attached
-            to the message.
-        input_message_content (:class:`telegram.InputMessageContent`, optional): Content of the
-            message to be sent instead of the voice.
-        **kwargs (:obj:`dict`): Arbitrary keyword arguments.
+            message to be sent instead of the voice message.
 
     """
 
-    def __init__(self,
-                 id,
-                 voice_file_id,
-                 title,
-                 caption=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 parse_mode=DEFAULT_NONE,
-                 **kwargs):
+    def __init__(
+        self,
+        id: str,  # pylint: disable=W0622
+        voice_file_id: str,
+        title: str,
+        caption: str = None,
+        reply_markup: 'ReplyMarkup' = None,
+        input_message_content: 'InputMessageContent' = None,
+        parse_mode: Union[str, DefaultValue] = DEFAULT_NONE,
+        caption_entities: Union[Tuple[MessageEntity, ...], List[MessageEntity]] = None,
+        **_kwargs: Any,
+    ):
         # Required
         super().__init__('voice', id)
         self.voice_file_id = voice_file_id
@@ -75,5 +89,6 @@ class InlineQueryResultCachedVoice(InlineQueryResult):
         # Optionals
         self.caption = caption
         self.parse_mode = parse_mode
+        self.caption_entities = caption_entities
         self.reply_markup = reply_markup
         self.input_message_content = input_message_content
