@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,20 +16,27 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
+# pylint: disable=C0114, E0401, W0622
 try:
     import ujson as json
 except ImportError:
-    import json
+    import json  # type: ignore[no-redef]
+
 from base64 import b64decode
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Union, no_type_check
 
 from cryptography.hazmat.backends import default_backend
-from cryptography.hazmat.primitives.asymmetric.padding import OAEP, MGF1
+from cryptography.hazmat.primitives.asymmetric.padding import MGF1, OAEP
 from cryptography.hazmat.primitives.ciphers import Cipher
 from cryptography.hazmat.primitives.ciphers.algorithms import AES
 from cryptography.hazmat.primitives.ciphers.modes import CBC
-from cryptography.hazmat.primitives.hashes import SHA512, SHA256, Hash, SHA1
+from cryptography.hazmat.primitives.hashes import SHA1, SHA256, SHA512, Hash
 
-from telegram import TelegramObject, TelegramError
+from telegram import TelegramError, TelegramObject
+from telegram.utils.types import JSONDict
+
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class TelegramDecryptionError(TelegramError):
@@ -37,10 +44,15 @@ class TelegramDecryptionError(TelegramError):
     Something went wrong with decryption.
     """
 
-    def __init__(self, message):
-        super().__init__("TelegramDecryptionError: {}".format(message))
+    def __init__(self, message: Union[str, Exception]):
+        super().__init__(f"TelegramDecryptionError: {message}")
+        self._msg = str(message)
+
+    def __reduce__(self) -> Tuple[type, Tuple[str]]:
+        return self.__class__, (self._msg,)
 
 
+@no_type_check
 def decrypt(secret, hash, data):
     """
     Decrypt per telegram docs at https://core.telegram.org/passport.
@@ -67,9 +79,9 @@ def decrypt(secret, hash, data):
     digest.update(secret + hash)
     secret_hash_hash = digest.finalize()
     # First 32 chars is our key, next 16 is the initialisation vector
-    key, iv = secret_hash_hash[:32], secret_hash_hash[32:32 + 16]
+    key, init_vector = secret_hash_hash[:32], secret_hash_hash[32 : 32 + 16]
     # Init a AES-CBC cipher and decrypt the data
-    cipher = Cipher(AES(key), CBC(iv), backend=default_backend())
+    cipher = Cipher(AES(key), CBC(init_vector), backend=default_backend())
     decryptor = cipher.decryptor()
     data = decryptor.update(data) + decryptor.finalize()
     # Calculate SHA256 hash of the decrypted data
@@ -79,11 +91,12 @@ def decrypt(secret, hash, data):
     # If the newly calculated hash did not match the one telegram gave us
     if data_hash != hash:
         # Raise a error that is caught inside telegram.PassportData and transformed into a warning
-        raise TelegramDecryptionError("Hashes are not equal! {} != {}".format(data_hash, hash))
+        raise TelegramDecryptionError(f"Hashes are not equal! {data_hash} != {hash}")
     # Return data without padding
-    return data[data[0]:]
+    return data[data[0] :]
 
 
+@no_type_check
 def decrypt_json(secret, hash, data):
     """Decrypts data using secret and hash and then decodes utf-8 string and loads json"""
     return json.loads(decrypt(secret, hash, data).decode('utf-8'))
@@ -94,12 +107,12 @@ class EncryptedCredentials(TelegramObject):
     Telegram Passport Documentation for a complete description of the data decryption and
     authentication processes.
 
-    Attributes:
-        data (:class:`telegram.Credentials` or :obj:`str`): Decrypted data with unique user's
-            nonce, data hashes and secrets used for EncryptedPassportElement decryption and
-            authentication or base64 encrypted data.
-        hash (:obj:`str`): Base64-encoded data hash for data authentication.
-        secret (:obj:`str`): Decrypted or encrypted secret used for decryption.
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`data`, :attr:`hash` and :attr:`secret` are equal.
+
+    Note:
+        This object is decrypted only when originating from
+        :obj:`telegram.PassportData.decrypted_credentials`.
 
     Args:
         data (:class:`telegram.Credentials` or :obj:`str`): Decrypted data with unique user's
@@ -109,13 +122,16 @@ class EncryptedCredentials(TelegramObject):
         secret (:obj:`str`): Decrypted or encrypted secret used for decryption.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
-    Note:
-        This object is decrypted only when originating from
-        :obj:`telegram.PassportData.decrypted_credentials`.
+    Attributes:
+        data (:class:`telegram.Credentials` or :obj:`str`): Decrypted data with unique user's
+            nonce, data hashes and secrets used for EncryptedPassportElement decryption and
+            authentication or base64 encrypted data.
+        hash (:obj:`str`): Base64-encoded data hash for data authentication.
+        secret (:obj:`str`): Decrypted or encrypted secret used for decryption.
 
     """
 
-    def __init__(self, data, hash, secret, bot=None, **kwargs):
+    def __init__(self, data: str, hash: str, secret: str, bot: 'Bot' = None, **_kwargs: Any):
         # Required
         self.data = data
         self.hash = hash
@@ -125,19 +141,10 @@ class EncryptedCredentials(TelegramObject):
 
         self.bot = bot
         self._decrypted_secret = None
-        self._decrypted_data = None
-
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-
-        data = super().de_json(data, bot)
-
-        return cls(bot=bot, **data)
+        self._decrypted_data: Optional['Credentials'] = None
 
     @property
-    def decrypted_secret(self):
+    def decrypted_secret(self) -> str:
         """
         :obj:`str`: Lazily decrypt and return secret.
 
@@ -153,18 +160,17 @@ class EncryptedCredentials(TelegramObject):
             # is the default for OAEP, the algorithm is the default for PHP which is what
             # Telegram's backend servers run.
             try:
-                self._decrypted_secret = self.bot.private_key.decrypt(b64decode(self.secret), OAEP(
-                    mgf=MGF1(algorithm=SHA1()),
-                    algorithm=SHA1(),
-                    label=None
-                ))
-            except ValueError as e:
+                self._decrypted_secret = self.bot.private_key.decrypt(
+                    b64decode(self.secret),
+                    OAEP(mgf=MGF1(algorithm=SHA1()), algorithm=SHA1(), label=None),
+                )
+            except ValueError as exception:
                 # If decryption fails raise exception
-                raise TelegramDecryptionError(e)
+                raise TelegramDecryptionError(exception) from exception
         return self._decrypted_secret
 
     @property
-    def decrypted_data(self):
+    def decrypted_data(self) -> 'Credentials':
         """
         :class:`telegram.Credentials`: Lazily decrypt and return credentials data. This object
             also contains the user specified nonce as
@@ -175,10 +181,10 @@ class EncryptedCredentials(TelegramObject):
                 private/public key but can also suggest malformed/tampered data.
         """
         if self._decrypted_data is None:
-            self._decrypted_data = Credentials.de_json(decrypt_json(self.decrypted_secret,
-                                                                    b64decode(self.hash),
-                                                                    b64decode(self.data)),
-                                                       self.bot)
+            self._decrypted_data = Credentials.de_json(
+                decrypt_json(self.decrypted_secret, b64decode(self.hash), b64decode(self.data)),
+                self.bot,
+            )
         return self._decrypted_data
 
 
@@ -189,7 +195,7 @@ class Credentials(TelegramObject):
         nonce (:obj:`str`): Bot-specified nonce
     """
 
-    def __init__(self, secure_data, nonce, bot=None, **kwargs):
+    def __init__(self, secure_data: 'SecureData', nonce: str, bot: 'Bot' = None, **_kwargs: Any):
         # Required
         self.secure_data = secure_data
         self.nonce = nonce
@@ -197,7 +203,9 @@ class Credentials(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['Credentials']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
@@ -234,20 +242,22 @@ class SecureData(TelegramObject):
             temporary registration.
     """
 
-    def __init__(self,
-                 personal_details=None,
-                 passport=None,
-                 internal_passport=None,
-                 driver_license=None,
-                 identity_card=None,
-                 address=None,
-                 utility_bill=None,
-                 bank_statement=None,
-                 rental_agreement=None,
-                 passport_registration=None,
-                 temporary_registration=None,
-                 bot=None,
-                 **kwargs):
+    def __init__(
+        self,
+        personal_details: 'SecureValue' = None,
+        passport: 'SecureValue' = None,
+        internal_passport: 'SecureValue' = None,
+        driver_license: 'SecureValue' = None,
+        identity_card: 'SecureValue' = None,
+        address: 'SecureValue' = None,
+        utility_bill: 'SecureValue' = None,
+        bank_statement: 'SecureValue' = None,
+        rental_agreement: 'SecureValue' = None,
+        passport_registration: 'SecureValue' = None,
+        temporary_registration: 'SecureValue' = None,
+        bot: 'Bot' = None,
+        **_kwargs: Any,
+    ):
         # Optionals
         self.temporary_registration = temporary_registration
         self.passport_registration = passport_registration
@@ -264,14 +274,18 @@ class SecureData(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['SecureData']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data['temporary_registration'] = SecureValue.de_json(data.get('temporary_registration'),
-                                                             bot=bot)
-        data['passport_registration'] = SecureValue.de_json(data.get('passport_registration'),
-                                                            bot=bot)
+        data['temporary_registration'] = SecureValue.de_json(
+            data.get('temporary_registration'), bot=bot
+        )
+        data['passport_registration'] = SecureValue.de_json(
+            data.get('passport_registration'), bot=bot
+        )
         data['rental_agreement'] = SecureValue.de_json(data.get('rental_agreement'), bot=bot)
         data['bank_statement'] = SecureValue.de_json(data.get('bank_statement'), bot=bot)
         data['utility_bill'] = SecureValue.de_json(data.get('utility_bill'), bot=bot)
@@ -312,15 +326,17 @@ class SecureValue(TelegramObject):
 
     """
 
-    def __init__(self,
-                 data=None,
-                 front_side=None,
-                 reverse_side=None,
-                 selfie=None,
-                 files=None,
-                 translation=None,
-                 bot=None,
-                 **kwargs):
+    def __init__(
+        self,
+        data: 'DataCredentials' = None,
+        front_side: 'FileCredentials' = None,
+        reverse_side: 'FileCredentials' = None,
+        selfie: 'FileCredentials' = None,
+        files: List['FileCredentials'] = None,
+        translation: List['FileCredentials'] = None,
+        bot: 'Bot' = None,
+        **_kwargs: Any,
+    ):
         self.data = data
         self.front_side = front_side
         self.reverse_side = reverse_side
@@ -331,7 +347,9 @@ class SecureValue(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['SecureValue']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
@@ -344,7 +362,7 @@ class SecureValue(TelegramObject):
 
         return cls(bot=bot, **data)
 
-    def to_dict(self):
+    def to_dict(self) -> JSONDict:
         data = super().to_dict()
 
         data['files'] = [p.to_dict() for p in self.files]
@@ -356,7 +374,7 @@ class SecureValue(TelegramObject):
 class _CredentialsBase(TelegramObject):
     """Base class for DataCredentials and FileCredentials."""
 
-    def __init__(self, hash, secret, bot=None, **kwargs):
+    def __init__(self, hash: str, secret: str, bot: 'Bot' = None, **_kwargs: Any):
         self.hash = hash
         self.secret = secret
 
@@ -366,24 +384,6 @@ class _CredentialsBase(TelegramObject):
 
         self.bot = bot
 
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-
-        return cls(bot=bot, **data)
-
-    @classmethod
-    def de_list(cls, data, bot):
-        if not data:
-            return []
-
-        credentials = list()
-        for c in data:
-            credentials.append(cls.de_json(c, bot=bot))
-
-        return credentials
-
 
 class DataCredentials(_CredentialsBase):
     """
@@ -399,10 +399,10 @@ class DataCredentials(_CredentialsBase):
         secret (:obj:`str`): Secret of encrypted data
     """
 
-    def __init__(self, data_hash, secret, **kwargs):
-        super().__init__(data_hash, secret, **kwargs)
+    def __init__(self, data_hash: str, secret: str, **_kwargs: Any):
+        super().__init__(data_hash, secret, **_kwargs)
 
-    def to_dict(self):
+    def to_dict(self) -> JSONDict:
         data = super().to_dict()
 
         del data['file_hash']
@@ -413,22 +413,22 @@ class DataCredentials(_CredentialsBase):
 
 class FileCredentials(_CredentialsBase):
     """
-        These credentials can be used to decrypt encrypted files from the front_side,
-        reverse_side, selfie and files fields in EncryptedPassportData.
+    These credentials can be used to decrypt encrypted files from the front_side,
+    reverse_side, selfie and files fields in EncryptedPassportData.
 
-        Args:
-            file_hash (:obj:`str`): Checksum of encrypted file
-            secret (:obj:`str`): Secret of encrypted file
+    Args:
+        file_hash (:obj:`str`): Checksum of encrypted file
+        secret (:obj:`str`): Secret of encrypted file
 
-        Attributes:
-            hash (:obj:`str`): Checksum of encrypted file
-            secret (:obj:`str`): Secret of encrypted file
-        """
+    Attributes:
+        hash (:obj:`str`): Checksum of encrypted file
+        secret (:obj:`str`): Secret of encrypted file
+    """
 
-    def __init__(self, file_hash, secret, **kwargs):
-        super().__init__(file_hash, secret, **kwargs)
+    def __init__(self, file_hash: str, secret: str, **_kwargs: Any):
+        super().__init__(file_hash, secret, **_kwargs)
 
-    def to_dict(self):
+    def to_dict(self) -> JSONDict:
         data = super().to_dict()
 
         del data['data_hash']
