@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,6 +17,8 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram ReplyKeyboardRemove."""
+from typing import Any
+
 from telegram import ReplyMarkup
 
 
@@ -27,29 +29,33 @@ class ReplyKeyboardRemove(ReplyMarkup):
     until a new keyboard is sent by a bot. An exception is made for one-time keyboards that are
     hidden immediately after the user presses a button (see :class:`telegram.ReplyKeyboardMarkup`).
 
-    Attributes:
-        remove_keyboard (:obj:`True`): Requests clients to remove the custom keyboard.
-        selective (:obj:`bool`): Optional. Use this parameter if you want to remove the keyboard
-            for specific users only.
-
     Example:
         A user votes in a poll, bot returns confirmation message in reply to the vote and removes
         the keyboard for that user, while still showing the keyboard with poll options to users who
         haven't voted yet.
 
+    Note:
+        User will not be able to summon this keyboard; if you want to hide the keyboard from
+        sight but keep it accessible, use :attr:`telegram.ReplyKeyboardMarkup.one_time_keyboard`.
+
     Args:
         selective (:obj:`bool`, optional): Use this parameter if you want to remove the keyboard
             for specific users only. Targets:
 
-            1) users that are @mentioned in the text of the Message object
-            2) if the bot's message is a reply (has reply_to_message_id), sender of the original
+            1) Users that are @mentioned in the text of the :class:`telegram.Message` object.
+            2) If the bot's message is a reply (has `reply_to_message_id`), sender of the original
                message.
 
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
+    Attributes:
+        remove_keyboard (:obj:`True`): Requests clients to remove the custom keyboard.
+        selective (:obj:`bool`): Optional. Use this parameter if you want to remove the keyboard
+            for specific users only.
+
     """
 
-    def __init__(self, selective=False, **kwargs):
+    def __init__(self, selective: bool = False, **_kwargs: Any):
         # Required
         self.remove_keyboard = True
         # Optionals
